'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

exports.querySubscriberDecorator = querySubscriberDecorator;

var _toGraphQL = require('react-relay/lib/toGraphQL');

var _toGraphQL2 = _interopRequireDefault(_toGraphQL);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function querySubscriberDecorator(onResponse) {
    var pendingQueries = {};

    function handlePendingQueries() {
        Object.keys(pendingQueries).forEach(function (name) {
            var queryRequests = pendingQueries[name];
            var fieldNames = Object.keys(queryRequests);

            Promise.all(fieldNames.map(function (fieldName) {
                return queryRequests[fieldName];
            })).then(function (results) {
                var serverResult = {};
                var variables = {};

                for (var i = 0; i < fieldNames.length; i++) {
                    var data = results[i].response;

                    var fieldName = fieldNames[i];
                    var query = queryRequests[fieldName].getQuery();
                    variables = _extends({}, variables, query.getVariables());
                    serverResult[fieldName] = {
                        query: _toGraphQL2.default.Query(query),
                        data: data
                    };
                }

                onResponse(name, variables, serverResult);
            });
        });
        pendingQueries = {};
    }

    return function (queryRequest) {
        var query = queryRequest.getQuery();
        var concreteQuery = query.getConcreteQueryNode();
        var name = concreteQuery.name,
            fieldName = concreteQuery.fieldName;


        pendingQueries[name] = pendingQueries[name] || {};
        pendingQueries[name][fieldName] = queryRequest;

        // Delay handling the queries until a tick later so they can be grouped by query name
        setImmediate(handlePendingQueries);
    };
}