'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _classic = require('react-relay/classic');

var _classic2 = _interopRequireDefault(_classic);

var _withRelayRenderer = require('./withRelayRenderer');

var _withRelayRenderer2 = _interopRequireDefault(_withRelayRenderer);

var _withRelayHelpers = require('./withRelayHelpers');

var _withRelayHelpers2 = _interopRequireDefault(_withRelayHelpers);

var _createRelayContainer = require('./createRelayContainer');

var _createRelayContainer2 = _interopRequireDefault(_createRelayContainer);

var _utils = require('../../utils');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function getComponentName(Component) {
    if (Component.WrappedComponent) {
        return getComponentName(Component.WrappedComponent);
    }
    var name = Component.displayName || Component.name || Component.constructor.name;
    if (process.env.NODE_ENV !== "production" && name.indexOf('(') > -1) {
        throw Error('Auto generated component name has brackets in it - this is probably due to a higher order component. You will either want to change the order of your higher order components or provide a name option to withRelayQuery.');
    }
    return name;
}

exports.default = function (_ref) {
    var name = _ref.name,
        params = _ref.params,
        query = _ref.query,
        initialVariables = _ref.initialVariables,
        forceFetch = _ref.forceFetch,
        ttl = _ref.ttl,
        isomorphic = _ref.isomorphic,
        _ref$withHelpers = _ref.withHelpers,
        withHelpers = _ref$withHelpers === undefined ? false : _ref$withHelpers;
    return function (Component) {
        // Grab the name from the component if it is not passed in manually
        name = name || getComponentName(Component);

        // Split out queries and fragments from query
        var queries = {};
        var fragments = {};
        (Array.isArray(query) ? query : [query]).forEach(function (query) {
            var _splitQuery = (0, _utils.splitQuery)(query, name),
                routeQuery = _splitQuery.routeQuery,
                fragment = _splitQuery.fragment;

            queries[routeQuery.fieldName] = function () {
                return routeQuery;
            };
            fragments[routeQuery.fieldName] = function () {
                return fragment;
            };
        });

        // Create a Relay container
        var Container = Component;
        if (withHelpers) {
            Container = (0, _withRelayHelpers2.default)()(Container);
        }
        Container = (0, _createRelayContainer2.default)({ fragments: fragments, initialVariables: initialVariables })(Container);

        // Prepare the query config
        var queryConfig;
        if (typeof params === 'function') {
            var queryConfigName = name;
            var paramsCallback = params;
            queryConfig = function queryConfig(props) {
                return {
                    name: queryConfigName,
                    queries: queries,
                    params: paramsCallback(props)
                };
            };
        } else {
            queryConfig = {
                name: name,
                queries: queries,
                params: params || {}
            };
        }

        return (0, _withRelayRenderer2.default)({
            forceFetch: forceFetch,
            ttl: ttl,
            isomorphic: isomorphic,
            queryConfig: queryConfig
        })(Container);
    };
};