'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _classic = require('react-relay/classic');

var _classic2 = _interopRequireDefault(_classic);

var _requestObject = require('./requestObject');

var _requestObject2 = _interopRequireDefault(_requestObject);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var getUniqueRequestID = function getUniqueRequestID(request) {
    return request.getID ? request.getID() : request.getVariables().input_0.clientMutationId;
};

var ClientNetworkLayer = function () {
    function ClientNetworkLayer() {
        var _this = this;

        var _ref = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {},
            _ref$graphqlUrl = _ref.graphqlUrl,
            graphqlUrl = _ref$graphqlUrl === undefined ? '/graphql' : _ref$graphqlUrl,
            _ref$headers = _ref.headers,
            headers = _ref$headers === undefined ? {} : _ref$headers,
            _ref$withCredentials = _ref.withCredentials,
            withCredentials = _ref$withCredentials === undefined ? false : _ref$withCredentials,
            env = _ref.env;

        _classCallCheck(this, ClientNetworkLayer);

        this.sendQuery = function (queryRequest) {
            return _this.send(queryRequest.getID(), 'query', queryRequest);
        };

        this._storeRequest = function (xhr, id) {
            var removeSelf = function removeSelf() {
                _this.env._requests.delete(id);
            };

            var endOfLifeEvents = ['error', 'abort', 'loadend', 'timeout'];
            endOfLifeEvents.forEach(function (event) {
                xhr.addEventListener(event, removeSelf);
            });

            var requestObj = new _requestObject2.default({
                id: id,
                _xhr: xhr,
                removeSelf: removeSelf
            });
            _this.env._requests.set(id, requestObj);
        };

        this.url = graphqlUrl;
        this.env = env;
        this.withCredentials = withCredentials;
        this.headers = _extends({
            'X-Requested-With': 'XMLHttpRequest',
            'Content-Type': 'application/json',
            'Accept': 'application/json'
        }, headers);
    }

    _createClass(ClientNetworkLayer, [{
        key: 'supports',
        value: function supports() {}
    }, {
        key: 'sendQueries',
        value: function sendQueries(queryRequests) {
            return Promise.all(queryRequests.map(this.sendQuery));
        }
    }, {
        key: 'sendMutation',
        value: function sendMutation(mutationRequest) {
            var id = getUniqueRequestID(mutationRequest);
            return this.send(id, 'mutation', mutationRequest);
        }

        // We store the XHR as a RequestObject so that it can be retrieved and manipulated later

    }, {
        key: 'onError',


        // These methods allow you to handle the promise rejection in your subclass
        // while also allowing access to the underlying xhr
        value: function onError(xhr, resolve, reject, response) {
            var error = new Error(JSON.stringify(response.errors));
            reject(error);
        }
    }, {
        key: 'getHeaders',
        value: function getHeaders() {
            return this.headers;
        }
    }, {
        key: 'getUrl',
        value: function getUrl(id, requestType, request) {
            return this.url;
        } // eslint-disable-line no-unused-vars

    }, {
        key: 'onLoad',
        value: function onLoad(xhr, resolve, reject) {
            if (xhr.status >= 400) {
                reject(new Error("Request failed"));
                return;
            }

            if (!xhr.response) {
                reject(new Error("Invalid JSON response returned from server"));
                return;
            }

            var response = xhr.response;
            if (typeof response === 'string') {
                try {
                    response = JSON.parse(response);
                } catch (err) {
                    reject(new Error("Invalid JSON response returned from server"));
                    return;
                }
            }
            var _response = response,
                data = _response.data;

            if ('errors' in response || !data) {
                this.onError(xhr, resolve, reject, response);
            } else {
                resolve(data);
            }
        }
    }, {
        key: 'send',
        value: function send(id, requestType, request) {
            var _this2 = this;

            return new Promise(function (resolve, reject) {
                var xhr = new XMLHttpRequest();

                xhr.open('POST', _this2.getUrl(id, requestType, request), true);
                xhr.responseType = 'json';
                if (_this2.withCredentials) {
                    xhr.withCredentials = true;
                }

                xhr.addEventListener("load", function () {
                    _this2.onLoad(xhr, resolve, reject);
                });

                xhr.addEventListener("error", function () {
                    reject(new TypeError('Network request failed'));
                });

                xhr.addEventListener("abort", function () {
                    reject({ cancelled: true });
                });

                var headers = _this2.getHeaders();
                var files = request.getFiles && request.getFiles();

                var body;
                if (files) {
                    var form = new FormData();
                    form.append('id', id);
                    form.append('query', request.getQueryString());
                    form.append('variables', JSON.stringify(request.getVariables()));
                    Object.keys(files).forEach(function (filename) {
                        if (Array.isArray(files[filename])) {
                            files[filename].forEach(function (file) {
                                form.append(filename, file);
                            });
                        } else {
                            form.append(filename, files[filename]);
                        }
                    });
                    body = form;
                    delete headers['Content-Type'];
                } else {
                    body = JSON.stringify({
                        id: id,
                        query: request.getQueryString(),
                        variables: request.getVariables()
                    });
                }

                Object.keys(headers).forEach(function (name) {
                    xhr.setRequestHeader(name, headers[name]);
                });

                var xhrId = getUniqueRequestID(request);
                _this2._storeRequest(xhr, xhrId);

                xhr.send(body);
            }).then(function (data) {
                request.resolve({ response: data });
                return data;
            }).catch(function (err) {
                request.reject(err);
            });
        }
    }]);

    return ClientNetworkLayer;
}();

exports.default = ClientNetworkLayer;